﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using HugeFlow.Core;
using System.Windows.Controls.Primitives;
using HugeFlow.Utility;

namespace HugeFlow.Controls.ColorPicker
{
    /// <summary>
    /// 블렌드의 컬러픽커와 유사한 유저컨트롤입니다.
    /// </summary>
    public partial class BlendColorPicker : UserControl
    {
        #region enums

        /// <summary>
        /// 모드 도메인 (ARGB와 RGB)
        /// </summary>
        public enum Modes
        {
            ARGB = 0,
            RGB = 1
        }

        #endregion enums

        #region Properties

        // 드래그
        private bool _isClicked = false;
        private Point _oldPoint;

        /// <summary>
        /// 제목
        /// </summary>
        public string Title 
        {
            get { return tbTitle.Text; }
            set { tbTitle.Text = value; }
        }

        private Modes _mode;
        /// <summary>
        /// 모드 (ARGB와 RGB)
        /// </summary>
        public Modes Mode
        {
            get { return _mode; }
            set
            {
                _mode = value;
                switch (_mode)
                {
                    default:
                    case Modes.ARGB:
                        VisualStateManager.GoToState(this, "ARGBMode", false);
                        break;
                    case Modes.RGB:
                        VisualStateManager.GoToState(this, "RGBMode", false);
                        alphaAmount.ChangeAmount(255, false);
                        break;
                }

                UpdateColorCode();
            }
        }

        /// <summary>
        /// 색상을 변경할 타겟
        /// </summary>
        public IColorChangable ColorChangeTarget = null;

        private Color _color;
        /// <summary>
        /// 컬러픽커의 색상
        /// </summary>
        public Color Color
        {
            get { return _color; }
            set
            {
                if (_color == value)
                    return;

                LastColor = _color;

                _color = value;

                PostColorChanged();

                // 타겟이 있다면 모두 변경
                if (ColorChangeTarget != null)
                {
                    ColorChangeTarget.UpdateColor(_color);
                }

                // 외부에 이벤트로 알림
                OnColorChanged();
            }
        }

        /// <summary>
        /// 이전 색상
        /// </summary>
        private Color _lastColor;
        public Color LastColor
        {
            get { return _lastColor; }
            set
            {
                Color netColor = value;
                netColor.A = 0xFF;

                _lastColor = netColor;

                rcLast.Fill = new SolidColorBrush(_lastColor);
            }
        }

        /// <summary>
        /// 초기 색상
        /// </summary>
        private Color _initialColor;
        public Color InitialColor
        {
            get { return _initialColor; }
            private set
            {
                _initialColor = value;
                rcInitial.Fill = new SolidColorBrush(_initialColor);
            }
        }

        /// <summary>
        /// 팝업
        /// </summary>
        static Popup _popup;

        #endregion Properties

        #region Events

        /// <summary>
        /// 컬러가 변경되었음을 알리는 이벤트
        /// </summary>
        public event EventHandler ColorChanged;

        private void OnColorChanged()
        {
            if (ColorChanged == null)
                return;

            ColorChanged(this, null);
        }

        #endregion Events

        #region Constructors

        /// <summary>
        /// static 멤버 초기화를 위한 static 생성자
        /// </summary>
        static BlendColorPicker()
        {
            _popup = new Popup();
        }

        /// <summary>
        /// 기본 생성자
        /// </summary>
        public BlendColorPicker()
        {
            InitializeComponent();

            LinkEvents();
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// 색상 변경후 취해야 할 행동
        /// </summary>
        private void PostColorChanged()
        {
            rcCurrent.Fill = new SolidColorBrush(Color);

            redAmount.ChangeAmount((int)Color.R, true);
            greenAmount.ChangeAmount((int)Color.G, true);
            blueAmount.ChangeAmount((int)Color.B, true);
            alphaAmount.ChangeAmount((int)Color.A, true);

            UpdateAmountBaseColor();
            UpdateColorCode();
        }

        /// <summary>
        /// 컬러 스페이스를 업데이트합니다.
        /// </summary>
        private void UpdateColorSpace()
        {
            colorSpace.MotherColor = motherColorBar.Color;
            colorSpace.Color = Color;
        }

        /// <summary>
        /// 모색을 컬러픽커에 설정된 색상으로 업데이트합니다.
        /// </summary>
        private void UpdateMotherColor()
        {
            motherColorBar.Color = Color;

            UpdateColorSpace();
        }

        /// <summary>
        /// 텍스트 박스에 컬러코드 표시를 업데이트합니다.
        /// </summary>
        private void UpdateColorCode()
        {
            switch (Mode)
            {
                default:
                case Modes.ARGB:
                    tboxCode.Text = string.Format("#{0:X2}{1:X2}{2:X2}{3:X2}", Color.A, Color.R, Color.G, Color.B);
                    break;
                case Modes.RGB:
                    tboxCode.Text = string.Format("#{0:X2}{1:X2}{2:X2}", Color.R, Color.G, Color.B);
                    break;
            }
        }

        /// <summary>
        /// 이벤트를 연결합니다.
        /// </summary>
        private void LinkEvents()
        {
            Loaded += new RoutedEventHandler(BlendColorPicker_Loaded);
            redAmount.AmountChanged += new EventHandler(redAmount_AmountChanged);
            greenAmount.AmountChanged += new EventHandler(greenAmount_AmountChanged);
            blueAmount.AmountChanged += new EventHandler(blueAmount_AmountChanged);
            alphaAmount.AmountChanged += new EventHandler(alphaAmount_AmountChanged);

            motherColorBar.ColorChangedByUser += new EventHandler(motherColorBar_ColorChangedByUser);
            colorSpace.PointMoved += new EventHandler(colorSpace_PointMoved);
            this.KeyUp += new KeyEventHandler(BlendColorPicker_KeyUp);
        }

        /// <summary>
        /// 창을 닫습니다.
        /// </summary>
        private void Close()
        {
            _popup.IsOpen = false;
            ColorChangeTarget = null;
        }

        /// <summary>
        /// 컬러코드 값을 읽어들여 컬러픽커의 컬러 값을 변경합니다.
        /// </summary>
        private void ChangeColorByColorCode()
        {
            try
            {
                Color newColor = Color;
                newColor = ColorHelper.ToColor(tboxCode.Text);

                Color = newColor;
                UpdateMotherColor();
            }
            catch
            {
                // 원상복구
                UpdateColorCode();
            }
        }

        /// <summary>
        /// 새 컬러를 적용합니다.
        /// </summary>
        /// <param name="newColor"></param>
        private void UpdateAmountBaseColor()
        {
            redAmount.BaseColor = Color;
            greenAmount.BaseColor = Color;
            blueAmount.BaseColor = Color;
            alphaAmount.BaseColor = Color;
        }

        /// <summary>
        /// 컬러픽커를 화면 중앙에 띄웁니다.
        /// </summary>
        /// <param name="color"></param>
        public void Show(Color color)
        {
            Point center = new Point 
            {
                X = (Application.Current.Host.Content.ActualWidth - this.ActualWidth) / 2,
                Y = (Application.Current.Host.Content.ActualHeight - this.ActualHeight) / 2
            };
            Show(color, center);
        }

        /// <summary>
        /// 컬러픽커를 실버라이트 어플리케이션 왼쪽 상단을 기준으로 원하는 위치에 띄웁니다.
        /// </summary>
        /// <param name="color"></param>
        /// <param name="position"></param>
        public void Show(Color color, Point position)
        {
            if (_popup.IsOpen == true)
                return;

            tfTranslate.X = position.X;
            tfTranslate.Y = position.Y;

            InitialColor = color;
            Color = color;

            _popup.Child = this;
            _popup.IsOpen = true;
        }

        /// <summary>
        /// 컬러픽커를 실버라이트 어플리케이션 왼쪽 상단을 기준으로 원하는 위치에 띄웁니다.
        /// </summary>
        /// <param name="color"></param>
        /// <param name="colorChangable"></param>
        public void Show(Color color, IColorChangable colorChangable)
        {
            ColorChangeTarget = colorChangable;

            Show(color, colorChangable.AbsolutePosition);
        }

        #endregion Methods

        #region EventHandlers

        void BlendColorPicker_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Escape)
            {
                Close();
            }
        }

        void rcBackground_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            _isClicked = false;
            rcBackground.ReleaseMouseCapture();
        }

        void rcBackground_MouseMove(object sender, MouseEventArgs e)
        {
            if (_isClicked == false)
                return;

            Point current = e.GetPosition(this);

            tfTranslate.X += current.X - _oldPoint.X;
            tfTranslate.Y += current.Y - _oldPoint.Y;

            _oldPoint = current;
        }

        void rcBackground_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            this.Focus();

            _isClicked = true;
            _oldPoint = e.GetPosition(this);

            rcBackground.CaptureMouse();
        }

        void colorSpace_PointMoved(object sender, EventArgs e)
        {
            Color = colorSpace.Color;
        }

        void tboxCode_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter)
            {
                ChangeColorByColorCode();
            }
        }

        private void tboxCode_GotFocus(object sender, RoutedEventArgs e)
        {
            tboxCode.SelectAll();
        }

        void tboxCode_LostFocus(object sender, RoutedEventArgs e)
        {
            ChangeColorByColorCode();
        }

        void motherColorBar_ColorChangedByUser(object sender, EventArgs e)
        {
            colorSpace.MotherColor = motherColorBar.Color;

            Color = colorSpace.Color;
        }


        void rcInitial_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            Color = InitialColor;
            UpdateMotherColor();
        }

        void alphaAmount_AmountChanged(object sender, EventArgs e)
        {
            Color newColor = Color;

            newColor.A = (byte)alphaAmount.Amount;
            Color = newColor;
            UpdateMotherColor();
        }

        void blueAmount_AmountChanged(object sender, EventArgs e)
        {
            Color newColor = Color;

            newColor.B = (byte)blueAmount.Amount;
            Color = newColor;
            UpdateMotherColor();
        }

        void greenAmount_AmountChanged(object sender, EventArgs e)
        {
            Color newColor = Color;

            newColor.G = (byte)greenAmount.Amount;
            Color = newColor;
            UpdateMotherColor();
        }

        void redAmount_AmountChanged(object sender, EventArgs e)
        {
            Color newColor = Color;

            newColor.R = (byte)redAmount.Amount;
            Color = newColor;
            UpdateMotherColor();
        }

        void BlendColorPicker_Loaded(object sender, RoutedEventArgs e)
        {
            redAmount.AmountTarget = ColorAmountBar.AmountTargets.Red;
            greenAmount.AmountTarget = ColorAmountBar.AmountTargets.Green;
            blueAmount.AmountTarget = ColorAmountBar.AmountTargets.Blue;
            alphaAmount.AmountTarget = ColorAmountBar.AmountTargets.Alpha;

            UpdateMotherColor();

            this.Focus();
        }

        private void buttonClose_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        #endregion EventHandlers
    }
}
