﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Text.RegularExpressions;

namespace HugeFlow.Controls.ColorPicker
{
    /// <summary>
    /// R,G,B,A 중 하나의 양을 표시하고 변경할 수 있는 유저컨트를
    /// </summary>
    public partial class ColorAmountBar : UserControl
    {
        #region enums

        public enum AmountTargets
        {
            None = 0,
            Red = 1,
            Green = 2,
            Blue = 3,
            Alpha = 4
        };

        #endregion enums

        #region Properties

        /// <summary>
        /// 유저컨트롤이 표시할 값의 종류 (R,G,B,A)
        /// </summary>
        public AmountTargets AmountTarget = AmountTargets.None;

        private int _amount = -1;
        /// <summary>
        /// 양
        /// </summary>
        public int Amount
        {
            get { return _amount; }
            private set
            {
                ChangeAmount(value, false);
            }
        }
        
        private Color _baseColor = Colors.White;
        /// <summary>
        /// 기초색
        /// </summary>
        public Color BaseColor
        {
            get { return _baseColor; }
            set
            {
                _baseColor = value;
                UpdateGradient();
            }
        }

        /// <summary>
        /// 드래그 지원
        /// </summary>
        private double _mouseMoveDistance = 0;
        private bool _recordMouseMoveDistance = false;
        private Point _oldPosition;

        
        private bool _isEditable = false;
        /// <summary>
        /// 양을 키보드로 입력할 수 있는 상태의 변화
        /// </summary>
        public bool IsEditable
        {
            get { return _isEditable;  }

            private set
            {
                _isEditable = value;

                if (_isEditable == true)
                {
                    VisualStateManager.GoToState(this, "Edit", false);

                    if (FocusManager.GetFocusedElement() != tboxAmount)
                    {
                        tboxAmount.Focus();
                        tboxAmount.Text = Amount.ToString();
                        tboxAmount.SelectAll();
                    }
                }
                else
                {
                    VisualStateManager.GoToState(this, "NoEdit", false);
                }
            }
        }

        #endregion Properties

        #region Events

        /// <summary>
        /// 양값이 변했음을 알려주는 이벤트
        /// </summary>
        public event EventHandler AmountChanged;
        private void OnAmountChanged()
        {
            if (AmountChanged == null)
                return;

            AmountChanged(this, null);
        }

        #endregion Events

        #region Constructors

        /// <summary>
        /// 기본 생성자
        /// </summary>
        public ColorAmountBar()
        {
            InitializeComponent();

            // 이벤트를 연결합니다.
            LinkEvents();
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// 사용자의 입력에 의해서 양값의 업데이트를 시도합니다.
        /// </summary>
        private void TryUpdateAmountByUserInput()
        {
            int amount = 0;

            // 빈문자열이면 양을 0으로
            if (string.IsNullOrEmpty(tboxAmount.Text) == true)
            {
                Amount = 0;
                return;
            }

            Regex reg = new Regex("[0-9]{1,2,3}");

            if (reg.Match(tboxAmount.Text).Success == true)
            {
                // 정상적인 값이면 업데이트를 한다.
                if (Int32.TryParse(tboxAmount.Text, out amount) == true &&
                    (amount >= 0 && amount <= 255))
                {
                    Amount = amount;
                }
            } 
        }

        /// <summary>
        /// 이벤트를 연결합니다.
        /// </summary>
        private void LinkEvents()
        {
            LayoutRoot.MouseLeftButtonDown += new MouseButtonEventHandler(LayoutRoot_MouseLeftButtonDown);
            LayoutRoot.MouseMove += new MouseEventHandler(LayoutRoot_MouseMove);
            LayoutRoot.MouseLeftButtonUp += new MouseButtonEventHandler(LayoutRoot_MouseLeftButtonUp);
            tboxAmount.LostFocus += new RoutedEventHandler(tboxAmount_LostFocus);
            tboxAmount.KeyUp += new KeyEventHandler(tboxAmount_KeyUp);
        }

        /// <summary>
        /// 양값 텍스트 표시를 되돌립니다.
        /// </summary>
        private void RestoreAmount()
        {
            tboxAmount.Text = Amount.ToString();
        }

        /// <summary>
        /// 하단의 그라디언트 영역의 색상을 업데이트합니다.
        /// </summary>
        private void UpdateGradient()
        {
            Byte red = _baseColor.R;
            Byte green = _baseColor.G;
            Byte blue = _baseColor.B;
            Byte alpha = _baseColor.A;

            // 색띠 보이기
            if (AmountTarget == AmountTargets.None)
            {
                layoutGradient.Visibility = Visibility.Collapsed;
            }
            else
            {
                layoutGradient.Visibility = Visibility.Visible;
            }

            switch (AmountTarget)
            {
                case AmountTargets.None:
                default:
                    break;
                case AmountTargets.Red:
                    gStopLeft.Color = Color.FromArgb(alpha, 0x00, green, blue);
                    gStopRight.Color = Color.FromArgb(alpha, 0xFF, green, blue);
                    break;
                case AmountTargets.Green:
                    gStopLeft.Color = Color.FromArgb(alpha, red, 0x00, blue);
                    gStopRight.Color = Color.FromArgb(alpha, red, 0xFF, blue);
                    break;
                case AmountTargets.Blue:
                    gStopLeft.Color = Color.FromArgb(alpha, red, green, 0x00);
                    gStopRight.Color = Color.FromArgb(alpha, red, green, 0xFF);
                    break;
                case AmountTargets.Alpha:
                    gStopLeft.Color = Color.FromArgb(0x00, red, green, blue);
                    gStopRight.Color = Color.FromArgb(0xFF, red, green, blue);
                    break;
            }
        }

        /// <summary>
        /// 이벤트 발생을 컨트롤하면서 양값을 바꿉니다.
        /// </summary>
        /// <param name="value"></param>
        /// <param name="handled"></param>
        public void ChangeAmount(int value, bool handled)
        {
            if (value < 0 || value > 255)
            {
                throw new ArgumentOutOfRangeException();
            }

            if (_amount == value)
                return;

            _amount = value;

            tbAmount.Text = _amount.ToString();

            rcPosition.Width = (LayoutRoot.Width - (rcPosition.Margin.Left + rcPosition.Margin.Right)) * (double)_amount / (double)byte.MaxValue;

            if (handled == false)
            {
                OnAmountChanged();
            }
        }

        #endregion Methods


        #region EventHandlers

        void tboxAmount_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter)
            {
                TryUpdateAmountByUserInput();
                IsEditable = false;
            }
            else if (e.Key == Key.Escape)
            {
                RestoreAmount();
                this.Focus();
                IsEditable = false;
            }
        }

        void tboxAmount_LostFocus(object sender, RoutedEventArgs e)
        {
            TryUpdateAmountByUserInput();

            IsEditable = false;
        }

        void LayoutRoot_MouseMove(object sender, MouseEventArgs e)
        {
            if (_recordMouseMoveDistance == false)
                return;

            Point currentPosition = e.GetPosition(LayoutRoot);

            Point move = new Point(currentPosition.X - _oldPosition.X, currentPosition.Y - _oldPosition.Y);

            if (move.X != 0)
            {
                int newAmount = Amount + (int)move.X;

                if (newAmount >= 0 && newAmount <= 255)
                {
                    Amount = newAmount;
                }
            }

            _mouseMoveDistance += Math.Abs(move.X) + Math.Abs(move.Y);

            _oldPosition = currentPosition;
        }

        void LayoutRoot_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            _mouseMoveDistance = 0;
            _recordMouseMoveDistance = true;

            LayoutRoot.CaptureMouse();
            _oldPosition = e.GetPosition(LayoutRoot);
        }

        void LayoutRoot_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            _recordMouseMoveDistance = false;
            LayoutRoot.ReleaseMouseCapture();

            if (_mouseMoveDistance > 0)
                return;

            IsEditable = true;
        }

        #endregion EventHandlers
    }
}