﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.Diagnostics;

namespace HugeFlow.Controls.ColorPicker
{
    public partial class MotherColorBar : UserControl
    {
        #region Properties

        //////////////////////////////////////////////////////////////////////////////
        // 상수
        public const int _MOTHERCOLOR_MAX = 127;
        const int _MOTHERCOLOR_GROUP_COUNT = 7;

        /// <summary>
        /// 인덱스
        /// </summary>
        public int MotherColorIndex
        {
            get { return _nMotherColorIndex; }
            private set
            {
                if (_nMotherColorIndex == value)
                    return;

                _nMotherColorIndex = value;

                UpdateCursorPosition();
            }
        }
        private int _nMotherColorIndex = -1;


        static private List<Color> MotherColorList;

        private enum MotherColorGroup
        {
            RGB = 0,
            GRB = 1,
            GBR = 2,
            BGR = 3,
            BRG = 4,
            RBG = 5
        }

        static private int[] MotherColorGroupIndex = new int[_MOTHERCOLOR_GROUP_COUNT];

        private bool _isMotherColorClicked = false;

        /// <summary>
        /// 색상
        /// </summary>
        public Color Color
        {
            get { return MotherColorList[MotherColorIndex]; }
            set
            {
                int bestIndex = GetBestMotherColorIndex(value);

                if (bestIndex == 0 &&
                    MotherColorIndex == _MOTHERCOLOR_MAX)
                {
                    bestIndex = _MOTHERCOLOR_MAX;
                }

                MotherColorIndex = bestIndex;
            }
        }

        #endregion Properties

        #region Events

        public event EventHandler ColorChangedByUser;

        private void OnColorChangedByUser()
        {
            if (ColorChangedByUser == null)
                return;

            ColorChangedByUser(this, null);
        }

        #endregion Events


        #region Constructors

        static MotherColorBar()
        {
            BuildMotherColorList();
        }

        /// <summary>
        /// 기본 생성자
        /// </summary>
        public MotherColorBar()
        {
            // Required to initialize variables
            InitializeComponent();

            LinkEvents();
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// 이벤트를 연결합니다.
        /// </summary>
        private void LinkEvents()
        {
            cvMotherColorBar.MouseLeftButtonDown += new MouseButtonEventHandler(cvMotherColorBar_MouseLeftButtonDown);
            cvMotherColorBar.MouseLeftButtonUp += new MouseButtonEventHandler(cvMotherColorBar_MouseLeftButtonUp);
            cvMotherColorBar.MouseMove += new MouseEventHandler(cvMotherColorBar_MouseMove);
        }

        /// <summary>
        /// Mother Color List 생성
        /// </summary>
        static void BuildMotherColorList()
        {
            MotherColorList = new List<Color>();

            ////////////////////////////////////////////////////////////////////////////////////////
            // 128개의 Mother Color를 추가 21 * 6구간

            int nIndexCurrent = 0;

            // G 증가 RGB
            MotherColorGroupIndex[(int)MotherColorGroup.RGB] = nIndexCurrent;
            for (int i = 0; i < 21; i++)
            {
                byte byteAmount = (byte)Math.Ceiling(((double)i / (double)21) * 255); // 증가
                MotherColorList.Add(Color.FromArgb(0xFF, 255, byteAmount, 0));
                nIndexCurrent++;
            }

            // R 감소
            MotherColorGroupIndex[(int)MotherColorGroup.GRB] = nIndexCurrent;
            for (int i = 0; i < 21; i++)
            {
                byte byteAmount = (byte)Math.Ceiling(((double)(21 - i) / (double)21) * 255); // 감소
                MotherColorList.Add(Color.FromArgb(0xFF, byteAmount, 255, 0));
                nIndexCurrent++;
            }

            // B 증가
            MotherColorGroupIndex[(int)MotherColorGroup.GBR] = nIndexCurrent;
            for (int i = 0; i < 21; i++)
            {
                byte byteAmount = (byte)Math.Ceiling(((double)i / (double)21) * 255); // 증가
                MotherColorList.Add(Color.FromArgb(0xFF, 0, 255, byteAmount));
                nIndexCurrent++;
            }

            // G 감소
            MotherColorGroupIndex[(int)MotherColorGroup.BGR] = nIndexCurrent;
            for (int i = 0; i < 21; i++)
            {
                byte byteAmount = (byte)Math.Ceiling(((double)(21 - i) / (double)21) * 255); // 감소
                MotherColorList.Add(Color.FromArgb(0xFF, 0, byteAmount, 255));
                nIndexCurrent++;
            }

            // R 증가
            MotherColorGroupIndex[(int)MotherColorGroup.BRG] = nIndexCurrent;
            for (int i = 0; i < 21; i++)
            {
                byte byteAmount = (byte)Math.Ceiling(((double)i / (double)21) * 255); // 증가
                MotherColorList.Add(Color.FromArgb(0xFF, byteAmount, 0, 255));
                nIndexCurrent++;
            }

            // B 감소
            MotherColorGroupIndex[(int)MotherColorGroup.RBG] = nIndexCurrent;
            for (int i = 0; i < 21; i++)
            {
                byte byteAmount = (byte)Math.Ceiling(((double)(21 - i) / (double)21) * 255); // 감소
                MotherColorList.Add(Color.FromArgb(0xFF, 255, 0, byteAmount));
                nIndexCurrent++;
            }

            // Red 두번 더 넣어줌
            MotherColorList.Add(Color.FromArgb(0xFF, 255, 0, 0));
            MotherColorList.Add(Color.FromArgb(0xFF, 255, 0, 0));
        }


        static int FindMotherSpecial(Color color)
        {
            for (int i = 0; i < MotherColorList.Count; i++)
            {
                if (color == MotherColorList[i])
                    return i;
            }
            return -1;
        }

        static int GetBestMotherColorIndex(Color color)
        {
            double hue = ColorUtility.GetHue(color);

            int newIndex = (int)Math.Round(127 * hue / 360);

            return newIndex;
        }

        #endregion Methods

        #region EventHandlers

        void cvMotherColorBar_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            _isMotherColorClicked = true;
            cvMotherColorBar.CaptureMouse();

            MotherColorIndex = Math.Min((int)e.GetPosition(cvMotherColorBar).Y, _MOTHERCOLOR_MAX);

            OnColorChangedByUser();
        }

        private void UpdateCursorPosition()
        {
            cvMotherColorCursor.SetValue(Canvas.TopProperty, (double)MotherColorIndex);
        }

        void cvMotherColorBar_MouseMove(object sender, MouseEventArgs e)
        {
            if (_isMotherColorClicked == false)
                return;

            MotherColorIndex = Math.Max(Math.Min((int)e.GetPosition(cvMotherColorBar).Y, _MOTHERCOLOR_MAX), 0);

            OnColorChangedByUser();
        }

        void cvMotherColorBar_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            _isMotherColorClicked = false;
            cvMotherColorBar.ReleaseMouseCapture();

            MotherColorIndex = Math.Max(Math.Min((int)e.GetPosition(cvMotherColorBar).Y, _MOTHERCOLOR_MAX), 0);

            OnColorChangedByUser();
        }

        #endregion EventHandlers

    }
}